@@ -2,7 +2,7 @@
  * cwdaemon - morse sounding daemon for the parallel or serial port
  * Copyright (C) 2002 - 2005 Joop Stakenborg <pg4i@amsat.org>
  *                      and many authors, see the AUTHORS file.
- * Copyright (C) 2012 - 2015 Kamil Ignacak <acerion@wp.pl>
+ * Copyright (C) 2012 - 2024 Kamil Ignacak <acerion@wp.pl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Library General Public License for more details.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -26,24 +26,20 @@
  * (Rubini et al. Linux Device Driver Book)
  */
 
-#define _BSD_SOURCE /* S_IFMT and related. */
+#ifndef __FreeBSD__
+#define _DEFAULT_SOURCE /* S_IFMT, S_IFCHR, on Linux (Debian 12 and probably earlier). */
+#endif
 
 #include "config.h"
 
-# if HAVE_STDIO_H
-# include <stdio.h>
-#endif
 #if STDC_HEADERS
-# include <stdlib.h>
 # include <stddef.h>
+# include <stdlib.h>
 #else
 # if HAVE_STDLIB_H
 #  include <stdlib.h>
 # endif
 #endif
-#if HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 #if HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
 #endif
@@ -51,8 +47,8 @@
 # include <fcntl.h>
 #endif
 #ifdef HAVE_LINUX_PPDEV_H
-# include <linux/ppdev.h>
 # include <linux/parport.h>
+# include <linux/ppdev.h>
 #endif
 #ifdef HAVE_DEV_PPBUS_PPI_H
 # include <dev/ppbus/ppi.h>
@@ -65,7 +61,18 @@
 # include <sys/param.h>
 #endif
 
+#include <errno.h>
+#include <stdio.h>
+#include <string.h>
+#include <unistd.h>
+
 #include "cwdaemon.h"
+#include "log.h"
+#include "lp.h"
+#include "utils.h"
+
+
+
 
 /**
    \file lp.c - parport functions
@@ -81,60 +88,81 @@
    \return -1 if the device is not suitable for use as parallel port based keyer
    \return a file descriptor if the device is suitable for use as a parallel port based keyer
 */
-int dev_get_parport(const char *fname)
+int lp_probe_cwdevice(const char *fname)
 {
-       char nm[MAXPATHLEN];
-       struct stat st;
-       int fd, m;
+	char nm[MAXPATHLEN] = { 0 };
+	struct stat st = { 0 };
+	int m = 0;
 
-       m = snprintf(nm, sizeof(nm), "/dev/%s", fname);
-       if (m >= sizeof(nm))
-               return (-1);
-       if ((fd = open(nm, O_RDWR | O_NONBLOCK)) == -1)
-               return (-1);
-       if (fstat(fd, &st) == -1)
-               goto out;
-       if ((st.st_mode & S_IFMT) != S_IFCHR)
-               goto out;
-       if (ioctl(fd, PPGETMODE, &m) == -1)
-               goto out;
-       return (fd);
+	int retv = build_full_device_path(nm, sizeof (nm), fname);
+	if (0 != retv) {
+		log_message(LOG_ERR, "Can't build path of lp device from [%s]: %s", fname, strerror(-retv));
+		return -1;
+	}
+	int fd = open(nm, O_RDWR | O_NONBLOCK);
+	if (fd == -1) {
+		log_message(LOG_ERR, "open() failed for lp device [%s]: %s", nm, strerror(errno)); // TODO (acerion) 2023.04.25: change this to NOTICE
+		return (-1);
+	}
+	if (fstat(fd, &st) == -1) {
+		log_message(LOG_ERR, "fstat() failed for lp device [%s]: %s", nm, strerror(errno));
+		goto out;
+	}
+	if ((st.st_mode & S_IFMT) != S_IFCHR) {
+		log_message(LOG_ERR, "lp device [%s] is not character device", nm);
+		goto out;
+	}
+	if (ioctl(fd, PPGETMODE, &m) == -1) {
+		log_message(LOG_ERR, "ioctl(PPGETMODE) failed for lp device [%s]: %s", nm, strerror(errno));
+		goto out;
+	}
+	return (fd);
 out:
-       close(fd);
-       return (-1);
+	close(fd);
+	return (-1);
 }
 
 #elif defined(HAVE_DEV_PPBUS_PPI_H)    /* FreeBSD (ppbus/ppi) */
 
-int dev_get_parport(const char *fname)
+int lp_probe_cwdevice(const char *fname)
 {
-       char nm[MAXPATHLEN];
-       struct stat st;
-       unsigned char c;
-       int fd, m;
+	char nm[MAXPATHLEN] = { 0 };
+	struct stat st = { 0 };
+	unsigned char c = 0;
+	int fd = 0;
 
-       m = snprintf(nm, sizeof(nm), "/dev/%s", fname);
-       if (m >= sizeof(nm))
-               return (-1);
-       if ((fd = open(nm, O_RDWR | O_NONBLOCK)) == -1)
-               return (-1);
-       if (fstat(fd, &st) == -1)
-               goto out;
-       if ((st.st_mode & S_IFMT) != S_IFCHR)
-               goto out;
-       if (ioctl(fd, PPISSTATUS, &c) == -1)
-               goto out;
-       return (fd);
+	int retv = build_full_device_path(nm, sizeof (nm), fname);
+	if (0 != retv) {
+		log_message(LOG_ERR, "Can't build path of lp device from [%s]: %s", fname, strerror(-retv));
+		return (-1);
+	}
+	if ((fd = open(nm, O_RDWR | O_NONBLOCK)) == -1) {
+		log_message(LOG_ERR, "open() failed for lp device [%s]: %s", nm, strerror(errno)); // TODO 2023.04.25: change this to NOTICE
+		return (-1);
+	}
+	if (fstat(fd, &st) == -1) {
+		log_message(LOG_ERR, "fstat() failed for lp device [%s]: %s", nm, strerror(errno));
+		goto out;
+	}
+	if ((st.st_mode & S_IFMT) != S_IFCHR) {
+		log_message(LOG_ERR, "lp device [%s] is not character device", nm);
+		goto out;
+	}
+	if (ioctl(fd, PPISSTATUS, &c) == -1) {
+		log_message(LOG_ERR, "ioctl(PPISSTATUS) failed for lp device [%s]: %s", nm, strerror(errno));
+		goto out;
+	}
+	return (fd);
 out:
-       close(fd);
-       return (-1);
+	close(fd);
+	return (-1);
 }
 
 #else                                  /* Fallback (nothing) */
 
-int dev_get_parport(const char *fname)
+int lp_probe_cwdevice(const char *fname)
 {
-       return (-1);
+	return (-1);
 }
 
 #endif
@@ -148,7 +176,7 @@ int dev_get_parport(const char *fname)
 static void
 parport_control (int fd, unsigned char controlbits, int values)
 {
-	struct ppdev_frob_struct frob;
+	struct ppdev_frob_struct frob = { 0 };
 	frob.mask = controlbits;
 	frob.val = values;
 
@@ -165,7 +193,7 @@ parport_control (int fd, unsigned char controlbits, int values)
 static void
 parport_control (int fd, unsigned char controlbits, int values)
 {
-	unsigned char val;
+	unsigned char val = 0;
 
 	if (ioctl (fd, PPIGCTRL, &val) == -1)
 	{
@@ -245,11 +273,7 @@ int
 lp_init (cwdevice * dev, int fd)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-	int mode;
-#endif
-
-#ifdef HAVE_LINUX_PPDEV_H
-	mode = PARPORT_MODE_PCSPP;
+	int mode = PARPORT_MODE_PCSPP;
 
 	if (ioctl (fd, PPSETMODE, &mode) == -1)
 	{
@@ -280,7 +304,7 @@ lp_init (cwdevice * dev, int fd)
 	parport_control (fd, STROBE, STROBE);
 #endif
 	dev->fd = fd;
-	dev->reset (dev);
+	dev->reset_pins_state(dev);
 	return 0;
 }
 
@@ -289,7 +313,7 @@ int
 lp_free (cwdevice * dev)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-	dev->reset (dev);
+	dev->reset_pins_state(dev);
 
 	/* Disable CW & PTT - /STROBE bit (pin 1) */
 	parport_control (dev->fd, PARPORT_CONTROL_STROBE, 0);
@@ -304,9 +328,7 @@ lp_free (cwdevice * dev)
 	return 0;
 }
 
-/* set to a known state */
-int
-lp_reset (cwdevice * dev)
+int lp_reset_pins_state(cwdevice * dev)
 {
 #if defined (HAVE_LINUX_PPDEV_H) || defined (HAVE_DEV_PPBUS_PPI_H)
 	lp_cw (dev, 0);
@@ -322,17 +344,19 @@ int
 lp_cw (cwdevice * dev, int onoff)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-	if (onoff == 1)
+	if (onoff == 1) {
 		parport_control (dev->fd, PARPORT_CONTROL_SELECT, 0);
-	else
+	} else {
 		parport_control (dev->fd, PARPORT_CONTROL_SELECT,
 				PARPORT_CONTROL_SELECT);
+	}
 #endif
 #ifdef HAVE_DEV_PPBUS_PPI_H
-	if (onoff == 1)
+	if (onoff == 1) {
 		parport_control (dev->fd, SELECTIN, 0);
-	else
+	} else {
 		parport_control (dev->fd, SELECTIN, SELECTIN);
+	}
 #endif
 	return 0;
 }
@@ -342,18 +366,20 @@ int
 lp_ptt (cwdevice * dev, int onoff)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-	if (onoff == 1)
+	if (onoff == 1) {
 		parport_control (dev->fd, PARPORT_CONTROL_INIT,
 				PARPORT_CONTROL_INIT);
-	else
+	} else {
 		parport_control (dev->fd, PARPORT_CONTROL_INIT, 0);
+	}
 #endif
 #ifdef HAVE_DEV_PPBUS_PPI_H
-	if (onoff == 1)
+	if (onoff == 1) {
 		parport_control (dev->fd, nINIT,
 				nINIT);
-	else
+	} else {
 		parport_control (dev->fd, nINIT, 0);
+	}
 #endif
 	return 0;
 }
@@ -368,7 +394,7 @@ lp_footswitch (cwdevice * dev)
 	footswitch = parport_read_data (dev->fd);
 	/* returns decimal 8 if pin 15 is high */
 #endif
-	return (int)((footswitch & 0x08) >> 3);
+	return ((footswitch & 0x08) >> 3);
 	/* bit 0=1 footswitch up, bit 0=0 footswitch down*/
 }
 
@@ -377,17 +403,19 @@ int
 lp_ssbway (cwdevice * dev, int onoff)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-	if (onoff == 1)		/* soundcard */
+	if (onoff == 1)	{   /* soundcard */
 		parport_control (dev->fd, PARPORT_CONTROL_AUTOFD,
 				PARPORT_CONTROL_AUTOFD);
-	else			/* microphone */
+	} else {            /* microphone */
 		parport_control (dev->fd, PARPORT_CONTROL_AUTOFD, 0);
+	}
 #endif
 #ifdef HAVE_DEV_PPBUS_PPI_H
-	if (onoff == 1)		/* soundcard */
+	if (onoff == 1)	{   /* soundcard */
 		parport_control (dev->fd, AUTOFEED, AUTOFEED);
-	else			/* microphone */
+	} else {            /* microphone */
 		parport_control (dev->fd, AUTOFEED, 0);
+	}
 #endif
 	return 0;
 }
